// $Id: date.C,v 1.15 2004/12/30 12:01:46 dvermeir Exp $

#include	<config.h>

// sys/time.h is needed for timeval
#define __EXTENSIONS__ // make solaris-2.7 read localtime_r from <time.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include	<string.h> // strlen()
#include	<iostream>

#include	"tostring.h"
#include	"date.h"
#include	"period.h"

// this routine is taken from gnu fileutils-3.16/lib
// it is in the public domain

extern "C" {
time_t get_date(const char*,time_t*);
}
namespace Dv {
namespace Util {

const std::string DateError::NAME("Dv::Util::DateError");

Date::Date() {
struct	timeval	t;
/*
*	t.tv_sec:	secs since 1/1/1970
*	t.tv_usec:	microseconds
*/
gettimeofday(&t, 0);
time_ = t.tv_sec;
}

Date::Date(time_t t): time_(t) {
}

Date::Date(unsigned int year, unsigned int month, unsigned int day,
   	    unsigned int hrs, unsigned int min, unsigned int sec): time_(0) {
struct tm 	tm;
//localtime_r(&time_,&tm);
tm.tm_isdst	= -1;
tm.tm_sec	= sec;
tm.tm_min	= min;
tm.tm_hour	= hrs; // hrs - 1, don't ask; it just works that way
tm.tm_mday	= day;
tm.tm_mon	= month;
tm.tm_year	= year-1900;
time_ 		= mktime(&tm);
/* debug
cerr << "tm.tm_isdst	= " << tm.tm_isdst 
     << ", tm.tm_sec	= " << tm.tm_sec 
     << ", tm.tm_min	= " << tm.tm_min 
     << ", tm.tm_hour	= " << tm.tm_hour 
     << ", tm.tm_mday	= " << tm.tm_mday 
     << ", tm.tm_mon	= " << tm.tm_mon 
     << ", tm.tm_year	= " << tm.tm_year 
     << endl;
*/
}

unsigned short
Date::year() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_year + 1900;
}

unsigned short
Date::month() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_mon;
}

unsigned short
Date::day() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_mday;
}

unsigned short
Date::hours() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_hour;
}

unsigned short
Date::minutes() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_min;
}

unsigned short
Date::seconds() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_sec;
}

unsigned short
Date::wday() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_wday;
}

unsigned short
Date::yday() const {
struct tm 	tm;
localtime_r(&time_,&tm);
return tm.tm_yday;
}

unsigned short
Date::yweek() const {
struct tm 	tm;
localtime_r(&time_,&tm);
static const int BUFSZ=10;
char buf[BUFSZ];
strftime(buf, BUFSZ, "%W", &tm);
short yw;
fromstring(yw, buf);
return yw;
}

void
Date::year(unsigned short yr) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_year = yr - 1900;
time_ 	= mktime(&tm);
}

void
Date::yday(unsigned short n) {
// kludge
if (yday() < n)
  while (yday()!=n)
    day(day()+1);
else
  while (yday()!=n)
    day(day()-1);
}

void
Date::month(unsigned short mo) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_mon = mo;
time_ 	= mktime(&tm);
}

void
Date::day(unsigned short dy) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_mday = dy;
time_ 	= mktime(&tm);
}

void
Date::hours(unsigned short hr) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_hour = hr;
time_ 	= mktime(&tm);
}

void
Date::minutes(unsigned short mins) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_min = mins;
time_ 	= mktime(&tm);
}

void
Date::seconds(unsigned short secs) {
struct tm 	tm;
localtime_r(&time_,&tm);
tm.tm_sec = secs;
time_ 	= mktime(&tm);
}

std::string
Date::str(const char* fmt) const {
static const char*	d_fmt = "%a %b %d %H:%M:%S %Y";
// %Z, i.e. time zone abbreviation, not included in the default
if (fmt==0)
  fmt = d_fmt;
const int size = strlen(fmt)*20;
char* buf = new char[size+1];

struct tm	tm;
localtime_r(&time_,&tm);

strftime(buf, size, fmt, &tm);
std::string	s(buf);
delete [] buf;
return s;
}

std::string
Date::gmt(const char* fmt) const {
static const char*	d_fmt = "%a %b %d %H:%M:%S %Z %Y";

if (fmt==0)
  fmt = d_fmt;
const int size = strlen(fmt)*20;
char* buf = new char[size+1];

struct tm	tm;
gmtime_r(&time_,&tm);

strftime(buf, size, fmt, &tm);
std::string	s(buf);
delete buf;
return s;
}


// Creates a date from a string according to the syntax accepted by parse().
Date::Date (const std::string& s) throw(DateError)
{
static const std::string error(": cannot parse \"");
time_ = get_date(s.c_str(),0);
if (time_<0)
  throw DateError(error+s+"\"");
}

void
Date::time(time_t t) {
time_ = t;
}

Date&
Date::operator+=(const Duration& u) {
*this = *this + u;
return *this;
}

}}

