#ifndef DVUTIL_DEBUG_H
#define DVUTIL_DEBUG_H
// $Id: debug.h,v 1.2 2003/08/12 08:42:15 dvermeir Exp $
#include <iostream>

namespace Dv {
namespace Util {
/** A simple class template for dumping an object to a stream.
 * This is useful if e.g. @a operator<<(ostream&, T) is 
 * already used. In such a case, one defines
 * a const member @a T::Debug(ostream&) and then uses it as in the
 * fragment below.
 * @code
 * T t;
 * cout << "T object=\n" << Debug<T>(t) << std::endl;
 * @endcode 
 */
template <typename T>
class Debug {
public:
  /** Constructor
   * @param t reference to object to be printed.
   */
  Debug(const T& t): t_(t) {}
  /** Print Debug version of object.
   * @param os output stream on which to print
   * @param d Debug wrapper around object
   * @return os
   */
  friend std::ostream& operator<<(std::ostream& os, const Debug<T>& d) {
    d.t_.debug(os); return os; 
    }
private:
  Debug(const Debug<T>&); // forbidden
  Debug<T> operator=(Debug<T>&); // forbidden
  const T& t_;
};
}}
#endif

