// $Id: includestream.C,v 1.5 2002/10/18 12:16:43 dvermeir Exp $
#include <string>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <config.h>
#include "includestream.h"
#include "file.h"

namespace Dv {
namespace Util {
using namespace std;

string
includestream::find(const string& filename, const directories& dirs) {
if (filename.size() == 0)
  return "";
if (filename[0] == '/') {
  File f(filename);
  if (f.exists())
    return filename;
  else
    return "";
  }
for (directories::const_iterator i=dirs.begin(); i!=dirs.end(); ++i) {
  Directory d(*i);
  if (d.exists()) {
    File f(d.file(filename));
    if (f.exists())
      return f.str();
    }
  }
return "";
}

includestream::includestream(istream& is, directories* dirs): istream(0), filter_(is,dirs) {
filterstreambuf<IncludeFilter> *sb(new filterstreambuf<IncludeFilter>(filter_));
rdbuf(sb);
clear(sb->filter().state());
}

includestream::includestream(istream& is, const string& dir): istream(0), filter_(is,dir) {
filterstreambuf<IncludeFilter> *sb(new filterstreambuf<IncludeFilter>(filter_));
rdbuf(sb);
clear(sb->filter().state());
}

includestream::~includestream() {
if (rdbuf())
  delete rdbuf();
}

includestream::IncludeFilter::IncludeFilter(istream& is, directories* dirs):
  dirs_(), pos_(string::npos) {
if (dirs)
  dirs_ = *dirs;
is_.push_back(&is);
}

includestream::IncludeFilter::IncludeFilter(istream& is, const string& dir):
  dirs_(), pos_(string::npos) {
dirs_.push_back(dir);
is_.push_back(&is);
}

includestream::IncludeFilter::~IncludeFilter() { 
}

void
includestream::IncludeFilter::pop(istream& is) { 
if (is_.size() == 0)
  throw runtime_error("__FILE__.__LINE__: cannot happen");
if (is_.back() != &is)
  throw runtime_error("__FILE__.__LINE__: cannot happen");
is_.pop_back();
// do not delete the last istream, which was
// provided by the caller
if (is_.size())
  delete &is;
}

int
includestream::IncludeFilter::get() { 
static string include("#include");
static string::size_type include_sz(include.size());

if (is_.size() == 0)
  return EOF;

istream& is = *(is_.back());

if (is) {
  if (pos_ ==string::npos) {
    if (! std::getline(is, line_)) {
      pop(is);
      return get();
      }
#ifdef HAVE_FUNC_STRING_COMPARE_IIS
    if (line_.compare(0, include_sz, include) == 0) {
#elif HAVE_FUNC_STRING_COMPARE_SII
    if (line_.compare(include, 0, include_sz) == 0) {
#else
#error "unknown signature string::compare"
#endif
      ifstream* ifs = 0;
      // push another include file
      pos_ = string::npos;
      string::size_type start=include_sz;
      while ( (start < line_.size()) && 
              ((line_[start] == ' ') || (line_[start] == '\t')) 
	    )
        ++start;
      if (start >= line_.size())
        throw runtime_error(line_ + ": syntax error");

      if (line_[start] == '"') {
        string::size_type end = line_.find_first_of('"', start+1);
        if (end == string::npos)
          throw runtime_error(line_ + ": syntax error");
        string filename(line_,start+1, end-start-1);
        if (filename.size() == 0)
          throw runtime_error(line_ + ": syntax error");
        ifs = new ifstream(filename.c_str());
        if (!*ifs)
          throw runtime_error(filename + ": cannot open");
        }
      else if (line_[start] == '<') {
        string::size_type end = line_.find_first_of('>', start+1);
        if (end == string::npos)
          throw runtime_error(line_ + ": syntax error");
        string filename(line_,start+1, end-start-1);
        if (filename.size() == 0)
          throw runtime_error(line_ + ": syntax error");
	string full_filename = find(filename, dirs_);
	if (full_filename.size() == 0)
          throw runtime_error(filename + ": does not exist");
        ifs = new ifstream(full_filename.c_str());
        if (!*ifs)
          throw runtime_error(full_filename + ": cannot open");
        }
      else
        throw runtime_error(line_ + ": syntax error");
      is_.push_back(ifs);
      return get();
      }
    pos_ = 0;
    }
  if (pos_ < line_.size() )
    return line_[pos_++];
  else if (pos_ == line_.size()) {
    pos_ = string::npos;
    return '\n';
    }
  }
else {
  pop(is);
  return get();
  }
return EOF;
}

ios::iostate
includestream::IncludeFilter::state() const {
if (is_.size() == 0)
  return ios::eofbit;
else
  return is_.back()->rdstate();
return ios::badbit;
}

}}
