// $Id: logstream.C,v 1.5 2002/10/18 11:41:26 dvermeir Exp $

#include <fstream>
#include "logstream.h"
#include "date.h"

namespace Dv {
namespace Util {

logstream::logstream(const std::string& path, const std::string& prefix): std::ostream(0), 
  filter_(path,prefix) {
Dv::Util::filterstreambuf<LogFilter>* sb(new Dv::Util::filterstreambuf<LogFilter>(filter_));
rdbuf(sb);
clear(sb->filter().state());
}

logstream::logstream(std::ostream& stream, const std::string& prefix): std::ostream(0),
  filter_(stream,prefix) {
Dv::Util::filterstreambuf<LogFilter>* sb(new Dv::Util::filterstreambuf<LogFilter>(filter_));
rdbuf(sb);
clear(sb->filter().state());
}

logstream::~logstream() {
if (rdbuf())
  delete rdbuf();
}

logstream::LogFilter::LogFilter(const std::string& path,const std::string& prefix):
  owner_(true), prefix_(prefix), lastnl_(true) {
stream_ = new std::ofstream(path.c_str(), std::ios::app);
}

logstream::LogFilter::LogFilter(std::ostream& os,const std::string& prefix): 
  owner_(false), prefix_(prefix), stream_(&os), lastnl_(true) {
}

logstream::LogFilter::~LogFilter() { 
if (owner_)
  delete stream_;
}

int
logstream::LogFilter::put(int c) { 
// alternative implementation: keep ostrstream buf and write only
// when line is complete; that should yield less chances for
// garbled output with concurrent write access.
bool ok(c!=EOF);
if (ok) {
  if (buf_.size()==0) {
    Date now;
    buf_.append(now.str("%Y-%m-%d %H:%M:%S ")).append(prefix_).append(": ");
    }
  if (c=='\n') {
    *stream_ << buf_ << std::endl;
    buf_.resize(0); // std::string::clear() not defined?
    }
  else
    buf_ += c;
  }
return (ok?c:EOF);
/*
bool ok(c!=EOF);
if (ok) {
  if (lastnl_) {
    Date	now;
    *stream_ << now.str("%Y-%m-%d %H:%M:%S ") << prefix_ << ": ";
    lastnl_ = false;
    }
  if (c=='\n') {
    *stream_ << endl;
    lastnl_ = true;
    }
  else
    stream_->put(c);
  ok = *stream_;
  }
return (ok?c:EOF);
*/
}

int
logstream::LogFilter::sync() { 
stream_->flush();
return (*stream_?0:-1);
}

std::ios::iostate
logstream::LogFilter::state() const {
if (stream_)
  return stream_->rdstate();
else
  return std::ios::badbit;
}


}}
