#ifndef DVUTIL_NOCOMMENTSTREAM_H
#define DVUTIL_NOCOMMENTSTREAM_H
// $Id: nocomment.h,v 1.8 2003/08/21 09:27:17 dvermeir Exp $

#include <string>
#include <iostream>
#include <dvutil/filterstreambuf.h>

/**
 * @file 
 * This files defines a class nocommentstream, which uses
 * a filterstreambuf to remove shell style comments.
 */

namespace Dv {
namespace Util {
/**
 * A nocommentstream is an istream that strips '#' comments.
 * All characters following (and including) a '#' are ignored
 * up to (but not including) the next newline character.
 *
 * Example usage:
 * @code
 * std::ifstream ifs("config.txt");
 * if (!ifs)
 *   throw std::runtime_error("config.txt: cannot open");
 * Dv::Util::nocommentstream nocifs(ifs);
 * std::string line;
 * while ( std::getline(nocifs, line) ) {
 *   // line does not contain comments
 *   }
 * @endcode
 */
class nocommentstream: public std::istream {
private:
  /** Implements Filter interface for filterstreambuf */
  class NoCommentFilter {
  public:
    NoCommentFilter(std::istream& is): is_(is) {}
    ~NoCommentFilter();
    int put(int c) { return EOF; }
    int get();
    int sync() { return -1; }
    void close() {}
    std::ios::iostate state() const;
  private:
    /** Underlying istream from where the filter takes its input.  */
    std::istream&  is_;
  };
public:
  /** Constructor.
   * @param is open stream from which original input is taken
   */
  nocommentstream(std::istream& is);
  /** Destructor.  */
  ~nocommentstream();
private:
  /** Filter */
  NoCommentFilter	filter_;
};
}}
#endif
