// $Id: ntostr.C,v 1.12 2002/10/18 11:41:26 dvermeir Exp $

#include "ntostr.h"
#include <stdexcept> // sprintf
#include <stdio.h> // sprintf
#include <stdlib.h> // strtod, strtol, strtoul
#include <ctype.h> // tolower

namespace Dv {
namespace Util {

std::string
ntostr(int i, const char*fmt) {
char    buf[20];
if (!fmt)
  fmt = "%d";
sprintf(buf,fmt,i);
return std::string(buf);
}

std::string
ntostr(unsigned int i,const char* fmt) {
char    buf[20];
if (!fmt)
  fmt = "%u";
sprintf(buf,fmt,i);
return std::string(buf);
}

std::string
ntostr(long i, const char* fmt) {
char    buf[20];

if (!fmt)
  fmt = "%ld";
std::sprintf(buf,fmt,i);
return std::string(buf);
}

std::string
ntostr(unsigned long i, const char* fmt) {
char    buf[20];

if (!fmt)
  fmt = "%lu";
sprintf(buf,"%lu",i);
return std::string(buf);
}

std::string
ntostr(double d,unsigned int precision, const char* fmt) {
char    buf[20];
char	myfmt[20];
if (fmt) {
  strncpy(myfmt,fmt,20);
  myfmt[20-1] = static_cast<char>(0);
  }
else
  sprintf(myfmt,"%%.%df",precision);
sprintf(buf,myfmt,d);
return std::string(buf);
}

std::string&
s2lower(std::string& s) {
char *pc(const_cast<char*>(s.data()));
std::string::size_type l(s.size());
for (std::string::size_type i=0;(i<l);++i,++pc)
  *pc=tolower(*pc);
return s;
}   

std::string&
s2upper(std::string& s) {
char *pc(const_cast<char*>(s.data()));
std::string::size_type l(s.size());
for (std::string::size_type i=0;(i<l);++i,++pc)
  *pc=toupper(*pc);
return s;
}   

char*
strim(char* pc) {
int     l = strlen(pc)-1;
while ((l>=0) && ((pc[l]==' ')||(pc[l]=='\n')||(pc[l]=='\r')))
  --l;
// { l >= -1 }
pc[++l] = 0;
while (*pc==' ')
  ++pc;
return pc;
}   

std::string&
strim(std::string& s) {
int     i = s.size()-1; // last valid position
int     n(i);
while ((s[i]==' ')||(s[i]=='\n')||(s[i]=='\r'))
  --i;
if (i<0)
  return s.erase(); // completely
if (i<n) // at least 1 char must be removed at the end
  s.erase(i+1);
i=0;
while ((s[i]==' ')||(s[i]=='\n')||(s[i]=='\r'))
  ++i;
if (i>0)
  s.erase(0,i);
return s; 
}

bool
cstr2long(const char* p,long& l,int base) {
if (!p)
  return false;
char*	end;
l = strtol(p,&end,base);
return end!=p;
}

bool
cstr2ulong(const char* p,unsigned long& l,int base) {
if (!p)
  return false;
char*	end;
l = strtoul(p,&end,base);
return end!=p;
}

bool
cstr2double(const char* p,double& d) {
if (!p)
  return false;
char*	end;
d = strtod(p,&end);
return end!=p;
}

Int::Int(const std::string& s,int base) throw (std::domain_error): int_(0) {
long	l;
if (cstr2long(s.c_str(),l,base))
  int_ = static_cast<int>(l);
else
  throw std::domain_error(std::string("Int::Int(")+s+"): not an int");
}

Long::Long(const std::string& s,int base) throw (std::domain_error): long_(0) {
if (cstr2long(s.c_str(),long_,base))
  ;
else
  throw std::domain_error(std::string("Long::Long(")+s+"): not a long");
}

Double::Double(const std::string& s) throw (std::domain_error): double_(0) {
if (cstr2double(s.c_str(),double_))
  ;
else
  throw std::domain_error(std::string("Double::Double(")+s+"): not a double");
}

}}
