// $Id: period.C,v 1.3 2002/10/18 12:45:06 dvermeir Exp $

#include <sstream>
#include "period.h"

namespace Dv {
namespace Util {
using namespace std;

static const size_t SECS_IN_MIN(60);
static const size_t SECS_IN_HOUR(SECS_IN_MIN * 60);
static const size_t SECS_IN_DAY(SECS_IN_HOUR * 24);
static const size_t SECS_IN_WEEK(SECS_IN_DAY * 7);

Duration
Duration::weeks(size_t wks, size_t days, size_t hrs, size_t mins, size_t secs) {
return Duration(0,0, 
	static_cast<unsigned long>(wks) * SECS_IN_WEEK 
	+ days * SECS_IN_DAY + hrs * SECS_IN_HOUR
	+ mins * SECS_IN_MIN + secs);
}

Duration
Duration::mins(size_t mins, size_t secs ) {
return weeks(0,0,0,mins,secs);
}

Duration
Duration::hrs(size_t hrs, size_t mins, size_t secs) {
return weeks(0,0,hrs,mins,secs);
}

Duration
Duration::days(size_t days, size_t hrs, size_t mins, size_t secs) {
return weeks(0,days,hrs,mins,secs);
}

Duration
Duration::months(size_t months) {
return Duration(0,months,0);
}

Duration
Duration::years(size_t years, size_t months) {
return Duration(years,months,0);
}

Duration
Duration::operator+(const Duration& d) const {
return Duration(years_ + d.years_, months_ + d.months_, seconds_ + d.seconds_);
}

Duration
Duration::operator+=(const Duration& d) {
*this = *this + d;
return *this;
}

string
Duration::str() const {
ostringstream	oss;
oss << *this;
return oss.str();
}

Period::Period(Date d1, Date d2) throw (runtime_error): start_(d1), size_(0,0,0) {
if (d2 < d1)
  throw runtime_error("Bad Period");

// Stupid and hurried and inefficient but simple code.
Date	dtmp(d1);
size_t  years=(d2.year() - d1.year());

while ( (dtmp + Duration::years(years)) > d2 ) 
  -- years;
  
size_ += Duration::years(years);

dtmp.year(dtmp.year() + years);

Duration delta = Duration::days(1);

size_t	ndays(0);

while (dtmp + delta <= d2) {
  ++ndays;
  dtmp += delta;
  }


size_ += Duration::days(ndays);

delta = Duration::hrs(1);

size_t	nhours(0);

while (dtmp + delta <= d2) {
  ++nhours;
  dtmp += delta;
  }

size_ += Duration::hrs(nhours);

delta = Duration::mins(1);
size_t	nmins(0);

while (dtmp + delta <= d2) {
  ++nmins;
  dtmp += delta;
  }

size_ += Duration::mins(nmins);

delta = Duration::mins(0,1);
size_t	nsecs(0);

while (dtmp + delta <= d2) {
  ++nsecs;
  dtmp += delta;
  }

size_ += Duration::mins(0,nsecs);

}

Date
Period::end() const {
return start_ + size_;
}

}}

using namespace Dv::Util;

ostream&
operator<<(ostream& os, const Duration& u) {
size_t days = u.seconds() / SECS_IN_DAY;
size_t secs = u.seconds() % SECS_IN_DAY;

if (u.years() || u.months() || days)
  os << u.years() << "-" << u.months() << "-" << days << " ";

size_t hrs = secs / SECS_IN_HOUR;
secs = secs % SECS_IN_HOUR;
size_t mins = secs / SECS_IN_MIN;
secs = secs % SECS_IN_MIN;

return os << hrs << ":" << mins << ":" << secs;
}

bool
operator==(const Duration& u1, const Duration& u2) {
return u1.years() == u2.years() && u1.months() == u2.months() &&
	u1.seconds() == u2.seconds();
}

Date
operator+(const Date& d, const Duration& u) {
Date	dnew(d);
if (u.years())
  dnew.year(d.year()+u.years());
if (u.months())
  dnew.month(d.month()+u.months());
if (u.seconds()) {
  size_t delta(u.seconds());
  size_t days = delta / SECS_IN_DAY;
  // keep same hours!
  while (days) {
    dnew.day(dnew.day()+1);
    -- days;
    dnew.hours(d.hours());
    dnew.minutes(d.minutes());
    dnew.seconds(d.seconds());
    }
  delta = delta % SECS_IN_DAY;
  size_t hours = delta / SECS_IN_HOUR;
  if (hours)
    dnew.hours(dnew.hours()+hours);
  delta = delta % SECS_IN_HOUR;
  if (delta)
    dnew.seconds(dnew.seconds() + delta);
  }
return dnew;
}
