// $Id: props.C,v 1.26 2002/10/19 08:25:25 dvermeir Exp $

#include  <string>
#include  <assert.h>
#include  <stdlib.h> // strtod(), strtol()
#include  <iterator>
#include  <iostream>
#include  <sstream>
#include  <map>
#include  <ctype.h>
#include  <string.h> // strchr()

#include  "ntostr.h"
#include  "props.h"

extern char**	environ; // global scope

namespace Dv {
namespace Util {
using namespace std;

/// Props -- property list
const string PropsException::NAME("Dv::Util::PropsException");

Props::Props(istream& is) {
is >> *this;
}

Props::~Props() {
for (propmap::iterator i=props_.begin(); i!=props_.end(); ++i)
  delete (*i).second;
}

Props&
Props::add(const string& key,const string& value) {
delete find(key);
props_[key] = new string(value);
return *this;
}

Props&
Props::append(const string& key,const string& new_value,const string& separator) {
propmap::const_iterator i=props_.find(key);
if (i==props_.end())
  add(key,new_value);
else 
  (*((*i).second) += separator) += new_value;
return *this;
}

Props&
Props::erase(const string& key) {
props_.erase(key);
return *this;
}

const string*
Props::find(const string& key) const {
propmap::const_iterator i=props_.find(key);
if (i==props_.end())
  return 0;
else
  return (*i).second;
}

string*
Props::found(const string& key) {
propmap::iterator i=props_.find(key);
if (i==props_.end())
  return 0;
else
  return (*i).second;
}

Props::ConstValue
Props::operator[](const string& key) const throw (PropsException) {
static const string OperationName("operator[");
const string*  ps = find(key);
if (ps)
  return ConstValue(*this,key,ps);
else
  throw PropsException(string("operator[")+key+"] undefined");
}

Props::Value
Props::operator[](const string& key) {
string*  ps = found(key);
return Value(*this,key,ps);
}

Props::ConstValue
Props::operator[](int i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](unsigned int i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](long i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](unsigned long i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](short i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](unsigned short i) const throw (PropsException) {
return (*this)[ntostr(i)];
}

Props::Value
Props::operator[](int i) {
return (*this)[ntostr(i)];
}

Props::Value
Props::operator[](unsigned int i) {
return (*this)[ntostr(i)];
}

Props::Value
Props::operator[](long i) {
return (*this)[ntostr(i)];
}

Props::Value
Props::operator[](unsigned long i) {
return (*this)[ntostr(i)];
}
Props::Value
Props::operator[](short i) {
return (*this)[ntostr(i)];
}

Props::Value
Props::operator[](unsigned short i) {
return (*this)[ntostr(i)];
}

Props::ConstValue
Props::operator[](const double& d) const throw (PropsException) {
return (*this)[ntostr(d)];
}

Props::Value
Props::operator[](const double& d) {
return (*this)[ntostr(d)];
}

istream&
operator>>(istream& is,Props & props) {
return props.read(is);
}

istream&
Props::read(istream& is,int n) {
char    c;
string  key,value;

for (int i=0; (i<n) || (n<0) ;++i) {
  // Assume we are at the beginning of a line, i.e. right after a '\n'
  // Skip comments and white space.
  bool skipping(true);
  while (skipping) {
    c = is.peek();
    if (c=='#') {
      string comment;
      getline(is,comment);
      }
    else if (isspace(c)) // too liberal, will also skip space in the
      is >> ws; // beginning of a line
    else
      skipping = false;
    }
  // Read a key.
  getline(is,key,'=');
  if (!is)
    return is; /// <<< this is the exit if n<0
  ostringstream  voss;
  // read value
  while (is.get(c))
    if (c=='\n') {
      if (is.get(c))
        if (c=='>')
          voss << '\n';
        else {
          is.putback(c);
          value = voss.str();
          add(key,value);
          break;
          }
      else {
        value = voss.str();
        add(key,value);
        break;
        }
      }
    else
      voss << c;
  }
return is;
}

ostream&
operator<<(ostream& os,const Props & props) {
for (Props::iterator i=props.begin();(i!=props.end());++i) {
  os << (*i).first << "=" ;
  const string *ps = (*i).second;
  assert(ps);
  for (unsigned int j=0;(j<ps->size());++j) {
    os << (*ps)[j];
    if ((*ps)[j]=='\n')
      os << '>';
    }
  os << endl;
  }
os << endl;
return os;
}

///////////////// Props::Value

ostream&
operator<<(ostream& os,const Props::Value& v) {
const string* ps(v.value_);
if (ps)
  return os << *ps;
else
  return os << "<null-props>";
}

Props&
Props::Value::operator>>(string& s) const throw (PropsException) {
if (!value_)
  throw PropsException(PropsException::NAME+": no value for key '"+key_+"'");
s.assign(*value_);
return props_;
}

Props&
Props::Value::operator>>(int& i) const throw (PropsException) {
string	s;
(*this)>>s;
long	l;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = int(l);
return props_;
}

Props&
Props::Value::operator>>(unsigned int& i) const throw (PropsException) {
string	s;
(*this)>>s;
unsigned long	l;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = int(l);
return props_;
}

Props&
Props::Value::operator>>(long& l) const throw (PropsException) {
string	s;
(*this)>>s;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an long");
return props_;
}

Props&
Props::Value::operator>>(unsigned long& l) const throw (PropsException) {
string	s;
(*this)>>s;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an long");
return props_;
}

Props&
Props::Value::operator>>(short& i) const throw (PropsException) {
string	s;
(*this)>>s;
long l;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not a short");
i = l;
return props_;
}

Props&
Props::Value::operator>>(unsigned short& i) const throw (PropsException) {
string	s;
(*this)>>s;
unsigned long l;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not a short");
i = l;
return props_;
}

Props&
Props::Value::operator>>(double& d) const throw (PropsException) {
string s;
(*this) >> s;
if (!cstr2double(s.c_str(),d))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not a double");
return props_;
}

Props::Value::operator string&() throw (PropsException) {
if (!value_)
  throw PropsException(PropsException::NAME+": no value for key '"+key_+"'");
return *value_;
};

Props::Value::operator int() const throw (PropsException) {
int i;
(*this) >> i;
return i;
};

Props::Value::operator unsigned int() const throw (PropsException) {
unsigned int i;
(*this) >> i;
return i;
};

Props::Value::operator long() const throw (PropsException) {
long l;
(*this) >> l;
return l;
};

Props::Value::operator unsigned long() const throw (PropsException) {
unsigned long l;
(*this) >> l;
return l;
};

Props::Value::operator short() const throw (PropsException) {
short l;
(*this) >> l;
return l;
};

Props::Value::operator unsigned short() const throw (PropsException) {
unsigned short l;
(*this) >> l;
return l;
};

Props::Value::operator double() const throw (PropsException) {
double d;
(*this) >> d;
return d;
};

Props&
Props::Value::operator=(const string& s) {
props_.add(key_,s);
return props_;
}

Props&
Props::Value::operator=(int i) {
props_.add(key_,ntostr(i));
return props_;
}

Props&
Props::Value::operator=(unsigned int i) {
props_.add(key_,ntostr(i));
return props_;
}

Props&
Props::Value::operator=(long l) {
props_.add(key_,ntostr(l));
return props_;
}

Props&
Props::Value::operator=(unsigned long l) {
props_.add(key_,ntostr(l));
return props_;
}

Props&
Props::Value::operator=(short l) {
props_.add(key_,ntostr(l));
return props_;
}

Props&
Props::Value::operator=(unsigned short l) {
props_.add(key_,ntostr(l));
return props_;
}

Props&
Props::Value::operator=(const double& d) {
props_.add(key_,ntostr(d));
return props_;
}

///////////////// Props::ConstValue

const Props&
Props::ConstValue::operator>>(string& s) const throw (PropsException) {
if (!value_)
  throw PropsException(PropsException::NAME+": no value for key '"+key_+"'");
s.assign(*value_);
return props_;
}

const Props&
Props::ConstValue::operator>>(int& i) const throw (PropsException) {
string	s;
(*this)>>s;
long	l;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = int(l);
return props_;
}

const Props&
Props::ConstValue::operator>>(unsigned int& i) const throw (PropsException) {
string	s;
(*this)>>s;
unsigned long	l;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = int(l);
return props_;
}

const Props&
Props::ConstValue::operator>>(long& l) const throw (PropsException) {
string	s;
(*this)>>s;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
return props_;
}

const Props&
Props::ConstValue::operator>>(unsigned long& l) const throw (PropsException) {
string	s;
(*this)>>s;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
return props_;
}

const Props&
Props::ConstValue::operator>>(short& i) const throw (PropsException) {
string	s;
(*this)>>s;
long l;
if (!cstr2long(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = l;
return props_;
}

const Props&
Props::ConstValue::operator>>(unsigned short& i) const throw (PropsException) {
string	s;
(*this)>>s;
unsigned long l;
if (!cstr2ulong(s.c_str(),l))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not an int");
i = l;
return props_;
}

const Props&
Props::ConstValue::operator>>(double& d) const throw (PropsException) {
string s;
(*this) >> s;
if (!cstr2double(s.c_str(),d))
  throw PropsException(PropsException::NAME+": value for key '"+key_+"' not a double");
return props_;
}

Props::ConstValue::operator const string&() const throw (PropsException) {
if (!value_)
  throw PropsException(PropsException::NAME+": no value for key '"+key_+"'");
return *value_;
};

Props::ConstValue::operator int() const throw (PropsException) {
int i;
(*this) >> i;
return i;
};

Props::ConstValue::operator unsigned int() const throw (PropsException) {
unsigned int i;
(*this) >> i;
return i;
};

Props::ConstValue::operator long() const throw (PropsException) {
long l;
(*this) >> l;
return l;
};

Props::ConstValue::operator unsigned long() const throw (PropsException) {
unsigned long l;
(*this) >> l;
return l;
};

Props::ConstValue::operator short() const throw (PropsException) {
short l;
(*this) >> l;
return l;
};

Props::ConstValue::operator unsigned short() const throw (PropsException) {
unsigned short l;
(*this) >> l;
return l;
};

Props::ConstValue::operator double() const throw (PropsException) {
double d;
(*this) >> d;
return d;
};


void
Props::substitute(const string& sin, string& sout,
                  string (*f)(const string&)) const throw (invalid_argument) {
static const string NAME("Props::substitute: ");
char lastc(0);
const char*	txt(sin.c_str());
unsigned int	size(sin.size());

for (unsigned int i=0;(i<size);) {
  char t(0);
  char c(txt[i]);
  switch (c=txt[i]) {
    case '%':
    case '$':
      t = c;
      if (lastc=='\\') {
        sout += c;
        ++i;
        }
      else if (i+3>=size) 
        // If $ is at pos i then the end of the var is at post at least
	// i+3, e.g. ${a}. Consequently, if i+3>=size, we have a syntax error.
        throw invalid_argument(NAME+"syntax error in \""+sin+"\"");
      else { // i+1, i+2 , i+3 are safe
        switch (txt[i+1]) {
          case '{': { // e.g. ${abc}
            int startname = i+2;
            unsigned int endname(startname);
            while ((endname<size)&&(txt[endname]!='}'))
	      ++endname;
	    if ((endname>=size)||(txt[endname]!='}'))
              throw invalid_argument(NAME+"syntax error in \""+sin+"\"");
            // now txt[startname] .. txt[endname-1] is name
            string name(txt+startname,endname-startname);
	    const string* value(find(name));
            if (value)
	      if ((t=='$') && (f))
	        sout += substitute(f(*value), f);
	      else
	        sout += substitute(*value, f);
	    else {
	      sout += t;
	      sout += '{';
	      sout += name;
	      sout += '}';
	      }
  	    c = char(0); // necessary to set lastc
            i = endname+1;
            }
	    break;
          default:
            sout += c;
	    ++i;
            break;
          }
        }
      break;
    case '\\':
      ++i;
      if (i==size)
        sout += c;
      else if ( (txt[i]!='$') && (txt[i]!='%') ) 
        sout += c;
      break;
    default:
      sout += c;
      ++i;
      break;
    }
  lastc = c;
  }
}

void
Props::substitute(istream& sin, ostream& sout,
                  string (*f)(const string&)) const throw (invalid_argument) {
static const string NAME("Props::substitute: ");
char lastc(0);

while (sin) {
  char t(0); // type: '$' or '%'
  char c;
  switch (c=sin.get()) {
    case EOF:
      if (lastc=='\\') 
        sout.put(c);
      break;
    case '%':
    case '$':
      t = c;
      if (lastc=='\\') 
        sout.put(c);
      else { 
        switch (sin.get()) {
          case '{': { // e.g. ${abc}
	    string name; // name of variable
	    while ((c=sin.get()) && c!=EOF && c!='\n' && c!= '}')
              name += c;
	    if ((c!='}')||(name.size()==0))
              throw invalid_argument(NAME+"syntax error in input stream");
	    const string* value(find(name));
            if (value)
	      if ((t=='$') && (f))
	        sout << substitute(f(*value), f);
	      else
	        sout << substitute(*value, f);
            else 
	      sout << t << '{' << name << '}';
  	    c = char(0); // necessary to set lastc
  	    t = char(0); 
            }
	    break;
          default:
	    if (c!=EOF)
              sout.put(c);
            break;
          }
        }
      break;
    case '\\':
      break;
    default:
      if (lastc=='\\')
        sout.put('\\');
      sout.put(c);
      break;
    }
  lastc = c;
  }
}

string
Props::substitute(const string& sin, string (*f)(const string&)) const
  throw (invalid_argument) {
string sout;
substitute(sin,sout,f);
return sout;
}

Props&
Props::select(const string& scope,Props& target) const {
const string prefix(scope+"::");
string::size_type l(prefix.size());
propmap::const_iterator	it(props_.lower_bound(prefix));
while (it!=props_.end()) {
  if ((*it).first.find(prefix)==0)
    target.add((*it).first.substr(l),*(*it).second);
  else
    break;
  ++it;
  }
return target;
}

Props&
Props::merge(const Props& source,const string& scope) throw (PropsException) {
if (&source==this) 
  throw PropsException(PropsException::NAME+": merge with self not supported");
string prefix("");
if (scope.size())
  prefix = scope+"::";
for (iterator i=source.begin(); i!=source.end(); ++i)
  if (prefix.size())
    add(prefix+(*i).first,*(*i).second);
  else
      add((*i).first,*(*i).second);
return *this;
}

Props::Props(const Props& source) throw (PropsException) {
if (&source==this)
  return;
for (iterator i=source.begin(); i!=source.end(); ++i)
  add((*i).first,*(*i).second);
return;
}

Props&
Props::operator=(const Props& source) throw (PropsException) {
if (&source==this)
  return *this;
clear();
for (iterator i=source.begin(); i!=source.end(); ++i)
  add((*i).first,*(*i).second);
return *this;
}

bool
Props::operator==(const Props& props) const {
propmap::const_iterator i = begin(); 
propmap::const_iterator iend = end(); 
propmap::const_iterator j = props.begin(); 
propmap::const_iterator jend = props.end(); 
while (i!=iend && j!=jend && (*i).first == (*j).first
	&& *(*i).second == *(*j).second) {
  ++i; ++j;
  }
return (i==iend && j == jend );
}

ostream&
Props::send(ostream& os) const {
return os << size() << endl << (*this);
}

istream&
Props::receive(istream& is) {
int n;
is >> n;
return read(is,n);
}

ostream&
operator<<(ostream& os,const Props::ConstValue& v) {
return os << *(v.value_);
}


Props&
env2props(Props& props) {
for (char** e=::environ;(*e);++e) {
  char* p(strchr(*e,'='));
  if (p) {
    string name(*e,p-*e);
    string value(p+1);
    props.add(name,value);
    }
  }
return props;
}

}}
