// $Id: pstream.C,v 1.5 2003/09/08 17:39:09 dvermeir Exp $
#include <stdio.h>
#include <errno.h>
#include <sys/wait.h>
#include "systemerror.h"
#include "fdstreambuf.h"

#include "pstream.h"

static inline FILE* file(void *pv) { return reinterpret_cast<FILE*>(pv); }


Dv::Util::pstream::pstream(const std::string& command, const std::string& mode)
  throw (std::runtime_error): std::iostream(0), file_(0), command_(command), mode_(mode) {
file_ = popen(command.c_str(), mode.c_str());
if (! file_) {
  if (errno)
    throw Dv::Util::SystemError(errno);
  throw std::runtime_error("popen fork() or pipe() failed");
  }
rdbuf(new Dv::Util::fdstreambuf(fileno(file(file_))));
}

size_t
Dv::Util::pstream::close() throw (std::runtime_error) {
if (! file_ )
  throw std::runtime_error("already closed");
int exit_code(0);
try {
  if (mode() == "w")
    flush();
  int status(pclose(file(file_)));
  file_ = 0;

  if (status == -1) {
    if (errno)
      throw Dv::Util::SystemError(errno);
    throw std::runtime_error("pclose failed");
    }
  if ( WIFEXITED(status) )
    exit_code = WEXITSTATUS(status);
  else
    throw std::runtime_error("command ended abnormally");
  }
catch (std::exception& e) {
  delete rdbuf();
  rdbuf(0);
  setstate(std::ios::failbit);
  throw;
  }
delete rdbuf();
rdbuf(0);
setstate(std::ios::failbit);
return exit_code;
}

Dv::Util::pstream::~pstream() throw (std::runtime_error) {
if ( file_ )
  close();
}

