// $Id: teestream.C,v 1.4 2002/10/18 10:10:01 dvermeir Exp $

#include "teestream.h"
#include <config.h>

namespace Dv {
namespace Util {

/////// Dv::Util::teestream

teestream::teestream(std::ostream& os1, std::ostream& os2): std::ostream(0), tee_(os1,os2) {
rdbuf(new Dv::Util::filterstreambuf<Tee>(tee_));
// inherit initial iostate from filter
clear(tee_.state());
}

teestream::teestream(const std::list<std::ostream*>& streams): std::ostream(0), tee_(streams) {
rdbuf(new Dv::Util::filterstreambuf<Tee>(tee_));
// inherit initial iostate from filter
clear(tee_.state());
}

/////// Dv::Util::teestream::Tee

teestream::Tee::Tee(std::ostream& os1,std::ostream& os2) {
streams_.push_back(&os1); streams_.push_back(&os2);
}

teestream::Tee::Tee(const std::list<std::ostream*>& streams): streams_(streams) {
}

teestream::Tee::~Tee() { 
sync(); 
}

int
teestream::Tee::put(int c) { 
bool ok(c!=EOF);
if (ok) {
  for (std::list<std::ostream*>::const_iterator i=streams_.begin(); i!=streams_.end(); ++i)
    if (!(**i).put(c))
      ok = false;
  }
return (ok?c:EOF);
}

int
teestream::Tee::sync() { 
bool ok(true);
for (std::list<std::ostream*>::const_iterator i=streams_.begin(); i!=streams_.end(); ++i)
  ok = ok && ((**i).rdbuf()->STREAMBUF_PUBSYNC() == 0);
return (ok?0:-1);
}

std::ios::iostate
teestream::Tee::state() const {
std::ios::iostate s(std::ios::goodbit); // we assume this means ``ok''
for (std::list<std::ostream*>::const_iterator i=streams_.begin(); i!=streams_.end(); ++i)
  s |= (**i).rdstate();
return s;
}

}}
