#ifndef DV_UTIL_TEESTREAM_H
#define DV_UTIL_TEESTREAM_H
// $Id: teestream.h,v 1.3 2002/10/18 09:50:14 dvermeir Exp $

#include <iostream>
#include "filterstreambuf.h"

/** \file
 * This files defines a class teestream, named after
 * the ``tee'' command, which illustrates the use
 * of Dv::Util::filterstreambuf.
 */
#include <list>

namespace Dv {
namespace Util {

/** 
 * A teestream object extends the unix ``tee'' command.
 * It is derived from ostream and initialized with
 * a list of ostream pointers. Anything written to
 * a teestream is written to all the underlying ostreams.
 */
class teestream: public std::ostream {
private:
  /**
   * Implements Filter interface for filterstreambuf
   */
  class Tee {
  public:
    Tee(std::ostream& os1, std::ostream& os2);
    Tee(const std::list<std::ostream*>& streams);
    ~Tee();
    int put(int c);
    int get() { return EOF; }
    int sync();
    void close() {}
    std::ios::iostate state() const;
  private:
    std::list<std::ostream*> streams_;
  };
public:
  /**
   * Special constructor for common case of 2 ostreams.
   * \param os1 first ostream to which output to teestream will be copied
   * \param os2 second ostream to which output to teestream will be copied
   */
  teestream(std::ostream& os1, std::ostream& os2);
  /**
   * General constructor takes a list of ostream pointers.
   * \param streams list of ostreams to which output to teestream will be
   *   copied
   */
  teestream(const std::list<std::ostream*>& streams);
private:
  Tee	tee_;
};

}}
#endif
