// $Id: test-file.C,v 1.1 2002/10/18 13:45:30 dvermeir Exp $

#include <stdexcept>
#include <iomanip>
#include <sstream>
#include "file.h"
#include "ntostr.h"

using namespace std;

const unsigned int MAXSUBDIRS(10);
const unsigned int MAXFILES(10);
const string TOPDIR("tfile-dir");
const string TMPDIR("tmpdir-");
const string TMPFILE("tmpfile.");
const string TMPLINK("tmplink.");
const string TMPSYMLINK("tmpsymlink.");
// names for moved files
const string TMP2DIR("tmp2dir-");
const string TMP2FILE("tmp2file.");
const string TMP2LINK("tmp2link.");
const string TMP2SYMLINK("tmp2symlink.");

int
main(int,char**) {
// this test has 2 parts: create directories and assorted files,
// then check that they exist and are of the right kind.
// then remove them
using namespace Dv::Util;
{
// file structure:
//	TOPDIR
//	  TMPDIR-m
//	    tmpfile.n
//	    tmplink.n --> tmpfile.n
//	    tmpsymlink.n --> tmpfile.n
try {
  Directory	here(Directory::pwd());
  Directory	dir(TOPDIR);
  try {
    dir.rmfr();
    if (dir.exists())
      throw runtime_error(TOPDIR+" exists and shouldn't");
    dir.mkdir();
  
    cerr << "Creating directories, files, links.." << endl;
    for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
      // d is ./tfile-dir/tmpdir-i
      Directory	d(dir.file(TMPDIR+ntostr(i)));
      if (d.exists())
        throw runtime_error(d.str()+" exists and shouldn't");
      d.mkdir();
      for (unsigned int j=0;(j<MAXFILES);++j) {
        // f is ./tfile-dir/tmpdir-i/tmpfile-j
        File	f(d.file(TMPFILE+ntostr(j)));
        if (f.exists())
          throw runtime_error(f.str()+" exists and shouldn't");
        if (!f.touch())
          throw runtime_error(f.str()+": touch failed");
        string	expected_fullname(d.str()+"/"+TMPFILE+ntostr(j));
        if (f.str()!=expected_fullname)
          throw runtime_error(f.str()+" !=  "+expected_fullname);
  
        // l is ./tfile-dir/tmpdir-i/tmplink-j
        File  l(d.file(TMPLINK+ntostr(j)));
        if (l.exists())
          throw runtime_error(l.str()+" exists and shouldn't");
        l.mklink(f.path());
        if (l.type()!=File::REGULAR)
          throw runtime_error(l.str()+" wrong type: "+
                  File::typestr(l.type())+" instead of REGULAR");
  
        // sl is ./tfile-dir/tmpdir-i/tmpsymlink-j
        File  sl(d.file(TMPSYMLINK+ntostr(j)));
        if (sl.exists())
          throw runtime_error(sl.str()+" exists and shouldn't");
        // const string oldpath(TOPDIR+"/"+TMPDIR+ntostr(i)+"/"+TMPFILE+ntostr(j));
        const string oldpath(TMPFILE+ntostr(j));
        if (!sl.mksymlink(oldpath))
          throw runtime_error(sl.str()+": cannot create symlink to "+oldpath);
        if (sl.type()!=File::SYMLINK)
          throw runtime_error(sl.str()+
                  " wrong type: "+File::typestr(sl.type())+
                  " instead of SYMLINK");
        
        }
      }
    
    cerr << "Moving directories, files, links.." << endl;
    for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
      // d is ./tfile-dir/tmpdir-i
      Directory	d(dir.file(TMPDIR+ntostr(i)));
      string new_name(dir.file(TMP2DIR+ntostr(i)));
      if (!d.mv(new_name))
        throw runtime_error(d.str()+": cannot move to "+new_name);
      // d is now ./tfile-dir/tmp2dir-i
      if (!d.exists())
        throw runtime_error(d.str()+" should exist after mv and doesn't");
      for (unsigned int j=0;(j<MAXFILES);++j) {
        {
        // f is ./tfile-dir/tmp2dir-i/tmpfile-j
        File	f(d.file(TMPFILE+ntostr(j)));
        if (!f.exists())
          throw runtime_error(f.str()+" should exist before mv and doesn't");
        string new_name(d.file(TMP2FILE+ntostr(j)));
        if (!f.mv(new_name))
          throw runtime_error(f.str()+": cannot move to "+new_name);
        }
        {
        File  l(d.file(TMPLINK+ntostr(j)));
        if (!l.exists())
          throw runtime_error(l.str()+" should exist before mv and doesn't");
        string new_name(d.file(TMP2LINK+ntostr(j)));
        if (!l.mv(new_name))
          throw runtime_error(l.str()+": could not move to "+new_name);
        if (l.type()!=File::REGULAR)
          throw runtime_error(l.str()+" wrong type");
        }
        {
        File  sl(d.file(TMPSYMLINK+ntostr(j)));
        if (!sl.exists())
          throw runtime_error(sl.str()+" should exist before mv and doesn't");
        string new_name(d.file(TMP2SYMLINK+ntostr(j)));
        if (!sl.mv(new_name))
          throw runtime_error(sl.str()+": could not move to "+new_name);
        if (sl.type()!=File::SYMLINK)
          throw runtime_error(sl.str()+" wrong type: "+File::typestr(sl.type()));
        }
        
        }
      }
    
    cerr << "Testing chmod(), mode().." << endl;
    for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
      Directory	d(dir.file(TMP2DIR+ntostr(i)));
      mode_t mode(File::RW_OWNER|File::RW_GROUP);
      for (unsigned int j=0;(j<MAXFILES);++j) {
        {
        File	f(d.file(TMP2FILE+ntostr(j)));
        f.chmod(mode);
        if (f.mode()!=mode) 
          throw runtime_error(f.str()+" chmod("+ntostr(mode)+") != "+ntostr(f.mode()));
        }
        {
        File	f(d.file(TMP2LINK+ntostr(j)));
        f.chmod(mode);
        if (f.mode()!=mode) 
          throw runtime_error(f.str()+" chmod("+ntostr(mode)+") != "+ntostr(f.mode()));
        }
        mode = (File::RW_OWNER);
        {
        File	f(d.file(TMP2SYMLINK+ntostr(j)));
        if (f.chmod(mode)) // should fail because operates on nonexistent underlying file
          throw runtime_error(f.str()+" chmod("+ntostr(mode)+") should have failed");
        }
        }
      }
    
    cerr << "Testing cd(), pwd().." << endl;
    for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
      Directory	d(dir.file(TMP2DIR+ntostr(i)));
      if (!d.cd()) {
        static const string errmsg("Cannot cd from ");
        throw runtime_error(errmsg+Directory::pwd()+" to "+d.str());
        }
      if (Directory::pwd() != d.str()) {
        static const string errmsg("Cd to ");
        }
      }
    
    cerr << "Testing relpath() .." << endl;
    for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
      Directory	dfrom(dir.file(TMP2DIR+ntostr(i)));
      if (!dfrom.cd()) {
        static const string errmsg("Cd from ");
        throw runtime_error(errmsg+Directory::pwd()+" to "+dfrom.str()+" failed");
        }
      for (unsigned int i=0;(i<MAXSUBDIRS);++i) {
        Directory	d(dir.file(TMP2DIR+ntostr(i)));
        for (unsigned int j=0;(j<MAXFILES);++j) {
          File	f(d.file(TMP2FILE+ntostr(j)));
          string frel(f.relpath());
          File    r(Directory::pwd()+"/"+frel);
          if (r!=f) {
            ostringstream oss;
            oss << "These files are different but should be the same:\n"
                 << f.str() << "\n"
                 << r.str() << "\n";
            throw runtime_error(oss.str());
            }
          }
        }
      }
    cerr << "Cleaning up.." << endl;
    here.cd();
    dir.rmfr();
    }
  catch (exception& e) {
    cerr << e.what() << endl;
    here.cd();
    dir.rmfr();
    return 1;
    }
  }
catch (exception& e) {
  cerr << "Could not even create " << TOPDIR << endl;
  cerr << e.what() << endl;
  return 1;
  }
}
return 0;
}
