// $Id: test-filterstreambuf.C.in,v 1.2 2003/08/05 13:05:37 dvermeir Exp $ 
#include <config.h>
#include <stdexcept>
#include <fstream>
#include <sstream>
#include "filterstreambuf.h"

class Filter {
public:
  Filter(std::ostream& os, const std::string& header): count_(0),
    header_(header), os_(os) {}
  int get();
  int put(int c);
  int sync();
private:
  size_t count_;
  std::string header_;
  std::ostream& os_;
};

int
Filter::get() {
return EOF;
}

int
Filter::put(int c) {
if (c!=EOF) {
  if (count_==0)
    os_ << header_;
  ++count_;
  return (os_.put(c) ? c : EOF);
  }
return EOF;
}

int
Filter::sync() {
return (os_.rdbuf()->STREAMBUF_PUBSYNC() == 0 ? 0 : -1);
}

class PrependStream: public std::ostream {
public:
  PrependStream::PrependStream(std::ostream& os, const std::string header): 
    std::ostream(0), filter_(os, header) {
    rdbuf(new Dv::Util::filterstreambuf<Filter>(filter_));
    }
private:
  Filter filter_;
};

int
main(int, char**) {
const std::string fn("./test-filterstreambuf.C");
const std::string HEADER("header\n");

try {
  std::string before;
  std::string after;
  {
    std::ifstream ifs(fn.c_str());
    if (!ifs)
      throw std::runtime_error(fn + ": cannot read");
    std::ostringstream oss;
    oss << ifs.rdbuf();
    before = oss.str();
  }
  {
    std::ifstream ifs(fn.c_str());
    if (!ifs)
      throw std::runtime_error(fn + ": cannot read");
    std::ostringstream oss;
    PrependStream pps(oss, HEADER);
    pps << ifs.rdbuf();
    after = oss.str();
  }
  if ( after != (HEADER + before) ) 
    throw std::runtime_error(HEADER + before + "\ndifferent from\n" + after);
  return 0;
  }
catch (std::exception& e) {
  std::cerr << e.what() << std::endl;
  return 1;
  }
}


