// $Id: test-pstream.C,v 1.4 2004/12/22 10:02:49 dvermeir Exp $

#include <sstream>
#include <fstream>

#include "pstream.h"
#include "file.h"
#include "tostring.h"

std::string
make_shell_with_exit(int n) throw (std::runtime_error) {
std::string fn("./pstream-test-shell.");
fn += Dv::Util::tostring(n);
std::ofstream ofs(fn.c_str());
if (! ofs) 
  throw std::runtime_error(fn + ": cannot create");
ofs << "#!/bin/sh\n" "exit " << n << "\n";
return fn;
}

int
main(int, char**) {
Dv::Util::pstream ips("ls", "r");
Dv::Util::pstream ops("wc -l", "w");
ops << ips.rdbuf();
std::cout << "ls exit code = " << ips.close() << std::endl;
std::cout << "wc exit code = " << ops.close() << std::endl;

Dv::Util::pstream ipspipe("cat test-pstream.C | wc -l", "r");
std::cout << ipspipe.rdbuf();
std::cout << "pipe exit code = " << ipspipe.close() << std::endl;

try {
  Dv::Util::pstream ips("./test-abort", "r");
  ips.close();
  std::cerr 
    << "error: pstream('test-abort') does not throw exception"
    << std::endl;
  return 1;
  }
catch (std::exception& e) {
  std::cerr << "expected exception: " <<  e.what() << std::endl;
  }

try {
  for (int i=0; i<5; ++i) {
    std::string fn(make_shell_with_exit(i));
    Dv::Util::File f(fn);
    if (! f.exists() )
      throw std::runtime_error(fn + ": does not exist");
    f.chmod();
    Dv::Util::pstream ips(fn,"r");
    int k(ips.close());
    if  (k != i)
      throw std::runtime_error(
        fn + ": expected " + Dv::Util::tostring(i) + ", got " + Dv::Util::tostring(k));
    std::cout << k << std::endl;
    f.rm();
    }
  }
catch (std::exception& e) {
  std::cerr << e.what() << std::endl;
  return 1;
  }
}

