#ifndef DV_THREAD_TOSTRING_H
#define DV_THREAD_TOSTRING_H
// $Id: tostring.h,v 1.6 2002/10/18 13:17:18 dvermeir Exp $
#include <string>
#include <sstream> // for ostringstream

namespace Dv {
namespace Util {
/**
 * A function that converts anything that can be written to
 * a stream to a string (using the ostringstream class).
 *
 *
 * \param t object for which a string representation is sought.
 * \return a string representation of t.
 */
template<typename T>
std::string tostring(const T& t) {
std::ostringstream oss;
oss << t;
return oss.str();
}

/**
 * A function that converts anything that can be read from
 * a stream to a string (using the istringstream class).
 *
 *
 * \param t object for which a string representation is sought.
 * \param s string representation from which <code>t</code> is to be computed
 * \return true iff success
 */
template<typename T>
bool
fromstring(T& t, const std::string& s) {
std::istringstream iss(s);
return bool(iss >> t);
}

}}
#endif
