// $Id: tty.C,v 1.4 2002/12/26 10:40:46 dvermeir Exp $

#include <errno.h> // errno
#include <sys/types.h> // open
#include <sys/stat.h> // open
#include <fcntl.h> // open
#include <unistd.h> // read
#include <termios.h> // tcgetattr etc.

#include "fdstreambuf.h"
#include "tty.h"

namespace Dv {
namespace Util {

class ttystreambuf: public fdstreambuf {
public:
  ttystreambuf(int fd, bool echo, time_t delay = 0, std::ostream* dbg = 0);
  ~ttystreambuf() {}
protected:
  /** Overrides version from fdstreambuf. */
  int read(char* buf, size_t n);
  int write(char* buf, size_t n);
private:
  struct termios term_default_;
  struct termios term_raw_;
};

ttystreambuf::ttystreambuf(int fd, bool echo, time_t delay, std::ostream* dbg): 
  fdstreambuf(fd, 1, 0, delay, false, dbg) {
tcgetattr(fd, &term_default_);

term_raw_ = term_default_;
term_raw_.c_lflag &= ~ICANON;
if (echo)
  term_raw_.c_lflag |= ECHO;
else
  term_raw_.c_lflag &= ~ECHO;
term_raw_.c_cc[VMIN] = 1;
term_raw_.c_cc[VTIME] = 0;
term_raw_.c_lflag &= ~ISIG;
}

int
ttystreambuf::read(char* buf, size_t n) {
if (tcsetattr(fd(), TCSANOW, &term_raw_) != 0) 
  return -1;
int k = fdstreambuf::read(buf, n);
if (tcsetattr(fd(), TCSANOW, &term_default_) != 0) 
  return -1;
return k;
}

int
ttystreambuf::write(char* buf, size_t n) {
if (tcsetattr(fd(), TCSANOW, &term_raw_) != 0) 
  return -1;
int k = fdstreambuf::write(buf, n);
if (tcsetattr(fd(), TCSANOW, &term_default_) != 0) 
  return -1;
return k;
}

Tty::Tty(bool echo, time_t delay, std::ostream* debug): std::iostream(0) {
int fd = open("/dev/tty", O_RDWR);
if (fd == -1) {
  setstate(std::ios::badbit);
  return;
  }

// echo_settings = no_echo_settings;
// echo_settings.c_lflag |= ECHO;

rdbuf(new ttystreambuf(fd,echo,delay,debug));
}

bool
Tty::timedout() const {
return static_cast<ttystreambuf*>(rdbuf())->timedout();
}

void
Tty::close() {
static_cast<ttystreambuf*>(rdbuf())->close();
}

void
Tty::reset() {
clear();
static_cast<ttystreambuf*>(rdbuf())->timedout(false);
}

Tty::~Tty() {
if (rdbuf())
  delete rdbuf();
rdbuf(0);
}

}}
