#ifndef DV_UTIL_TTY_H
#define DV_UTIL_TTY_H
// $Id: tty.h,v 1.5 2002/12/26 10:40:47 dvermeir Exp $

#include <string>
#include <iostream>

/** \file
 *  This file defines an iostream class ``Tty'' that provides rar
 *  unbuffered input and output to /dev/tty.
 */

namespace Dv {
namespace Util {

/** 
 * An iostream class that provides raw unbuffered input and output to
 * /dev/tty.
 */
class Tty: public std::iostream {
public:
  /** Open /dev/tty as an unbuffered stream.
   * \param echo if true, characters will be echoed
   * \param delay if nonzero, an input operation will fail, and the
   * member function timedout() will return true, if no character is
   * typed for delay millisecs.
   * \param debug pointer to stream on which debug information will be
   * written.
   */
  Tty(bool echo=false, time_t delay = 0, std::ostream* debug = 0);
  /**
   * Destructor.
   */
  ~Tty();
  /** \return true iff the last input operation timedout. This
   * condition can be reset using Tty::reset().
   */
  bool timedout() const;
  /** Clear the flags and set timedout to false. */
  void reset();
  /** Close the underlying file descriptor. */
  void close();
private:
  bool echo_;
};

}}
#endif
