// $Id: user.C,v 1.8 2003/10/29 13:49:08 dvermeir Exp $

#define	_XOPEN_SOURCE
#define __EXTENSIONS__ // for solaris-2.7
#include <config.h> // HAVE_GETSPNAM HAVE_GETPWNAM
#include <unistd.h> // crypt(), getuid()
#include <pwd.h>

#ifdef HAVE_GETSPNAM
#include <shadow.h>
#endif

#include "user.h"
#include "ntostr.h"

#include  <errno.h>  // errno
#include  <string.h>  // strerror()

namespace Dv {
namespace Util {
using namespace std;

/////////////////// UserError

const string UserError::NAME("Dv::Util::UserError");

/////////////////// User
User::User(): uid_(getuid()) {
if (getpwinfo(getpwuid(uid_))==0) {
  int e(errno);
  throw UserError("unexpected in User::User(): uid = "+ntostr(uid_)+strerror(e));
  }
}

User::User(uid_t uid) throw (UserError): uid_(uid) {
if (getpwinfo(getpwuid(uid_))==0) 
  throw UserError(ntostr(uid_)+": bad uid");
}

User::User(const string& name) throw (UserError): name_(name) {
if (getpwinfo(getpwnam(name_.c_str()))==0)
  throw UserError(name+": bad name");
}

void*
User::getpwinfo(void* p) {
if (p==0)
  return 0;
struct passwd*	pinfo = static_cast<passwd*>(p);
name_		= pinfo->pw_name;
full_name_	= pinfo->pw_gecos;
uid_		= pinfo->pw_uid;
gid_		= pinfo->pw_gid;
home_		= pinfo->pw_dir;		
shell_		= pinfo->pw_shell;		
return p;
}

bool
User::password(const string& pass) const {
#ifdef HAVE_GETSPNAM
struct spwd* pwdinfo = static_cast<struct spwd*>(getspnam(name_.c_str()));
if (pwdinfo) {
  string pwd(pwdinfo->sp_pwdp);
  return (pwd == ::crypt(pass.c_str(),pwd.c_str()));
  }
#else
#ifdef HAVE_GETPWNAM
struct passwd* pwdinfo = static_cast<struct passwd*>(getpwnam(name_.c_str()));
if (pwdinfo) {
  string pwd(pwdinfo->pw_passwd);
  return (pwd == ::crypt(pass.c_str(),pwd.c_str()));
  }
#endif
#endif
return false;
}



}}


